// This file is automatically generated by Make.bash.
// Any changes made to this file will be lost.
package cynosurex.library.TetherProxLibrary;
import cynosurex.application.TetherProx.R;
/* $Id: Request.java,v 1.9 2000/03/08 15:25:25 boyns Exp $ */

/*
 * Copyright (C) 1996-2000 Mark R. Boyns <boyns@doit.org>
 *
 * This file is part of Muffin.
 *
 * Muffin is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Muffin is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Muffin; see the file COPYING.  If not, write to the
 * Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.util.StringTokenizer;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.util.regex.Pattern;
import java.util.regex.Matcher;

/** @author Mark Boyns */
public class Request extends Message {
  private static Pattern httpRegex = Pattern.compile("^(http|https):", Pattern.CASE_INSENSITIVE);

  private String command = null;
  private String url = null;
  private ParsedURL parsedURL;
  private String protocol = null;
  private byte[] data = null;
  private Client client = null;
  private Hashtable log;
  private Vector logHeaders;

  Request(Client c) {
    client = c;
  }

  void read(InputStream in) throws IOException {
    statusLine = readLine(in);
    if (statusLine == null || statusLine.length() == 0) {
      throw new IOException("Empty request");
    }

    StringTokenizer st = new StringTokenizer(statusLine);
    command = (String) st.nextToken();
    url = (String) st.nextToken();
    if(!command.equalsIgnoreCase("connect")) {
      parsedURL = URLParser.parse(url);
    } else {
      parsedURL = URLParser.parse("https://"+url);
    }
    protocol = (String) st.nextToken();

    if (!url.startsWith("http")) {
      Matcher match = httpRegex.matcher(url);
      if (match.find()) {
        url = url.substring(match.start(),
            match.end()).toLowerCase()
            + url.substring(match.end());
      }
    }

    readHeaders(in);

    if ("POST".equals(command) || "PUT".equals(command)) {
      try {
        int n = Integer.parseInt(getHeaderField("Content-length"));
        data = new byte[n];
        int offset = 0;
        while (offset < data.length) {
          n = in.read(data, offset, data.length - offset);
          if (n < 0) {
            throw new IOException("Not enough " + command + " data");
          }
          offset += n;
        }
      }
      catch (NumberFormatException e) {
        System.out.println("Malformed or missing " + command + " Content-length");
      }
    }
  }

  public void write(OutputStream out)
      throws IOException {
    super.write(out);
    if (data != null) {
      out.write(data);
      out.flush();
    }
  }

  public String getRequest() {
    return statusLine;
  }

  public String getCommand() {
    return command;
  }

  public void setCommand(String command) {
    this.command = command;
  }

  public String getURL() {
    return url;
  }

  public void setURL(String url) {
    this.url = url;
  }

  public String getProtocol() {
    return protocol;
  }

  public void setProtocol(String protocol) {
    this.protocol = protocol;
  }

  public String getHost() {
    return parsedURL.getHost();
  }

  public int getPort() {
    return parsedURL.getPort() != null ? Integer.parseInt(parsedURL.getPort()) : 80;
  }

  public String getData() {
    if (data == null) {
      return null;
    }
    return new String(data);
  }

  public String getPath() {
    StringBuilder sb = new StringBuilder();
    String path = parsedURL.getPath();
    if(path == null) path ="/";
    sb.append(path);
    if(parsedURL.getQuery() != null) sb.append("?").append(parsedURL.getQuery());
    return sb.toString();
  }

  public String getDocument() {
    String path = getPath();
    int n = path.lastIndexOf('/');
    if (n == path.length() - 1) {
      n = path.lastIndexOf('/', n - 1);
    }
    if (n < 0) {
      return "/";
    } else {
      return path.substring(n + 1);
    }
  }

  public Client getClient() {
    return client;
  }

  public String getQueryString() {
    return parsedURL.getQuery();
  }

  public synchronized void addLogEntry(String header,
                                       String message) {
    if (log == null) {
      log = new Hashtable();
      logHeaders = new Vector();
    }

    Vector v = (Vector) log.get(header);
    if (log.get(header) == null) {
      v = new Vector();
      log.put(header, v);
      logHeaders.addElement(header);
    }
    v.addElement(message);
  }

  public Enumeration getLogHeaders() {
    return logHeaders != null ? logHeaders.elements() : null;
  }

  public Enumeration getLogEntries(String header) {
    return log != null ? ((Vector) log.get(header)).elements() : null;
  }
}
